require 'test/unit'
require 'behaviors'

require 'user'

class UserTest < Test::Unit::TestCase
  extend Behaviors

  def setup
  end

  should "be able set user name and age during construction"
  should "be able to get user name and age"
  should "be able to ask if a user is an adult"
  def test_DELETEME
  end
end
